@@ -60,7 +60,7 @@ from email.utils import parseaddr
 
 SUPPORTED_CERTAINTIES = ["certain", "confident", "likely", "possible", None]
 
-version_string = "0.1.22"
+version_string = "0.1.24"
 
 USER_AGENT = "upstream-ontologist/" + version_string
 # Too aggressive?
@@ -72,12 +72,24 @@ class Person:
 
     name: str
     email: Optional[str] = None
+    url: Optional[str] = None
 
     @classmethod
     def from_string(cls, text):
         text = text.replace(' at ', '@')
         text = text.replace('[AT]', '@')
-        if '<' in text:
+        if '(' in text and text.endswith(')'):
+            (p1, p2) = text[:-1].split('(', 1)
+            if p2.startswith('https://') or p2.startswith('http://'):
+                url = p2
+                if '<' in p1:
+                    (name, email) = parseaddr(p1)
+                    return cls(name=name, email=email, url=url)
+                return cls(name=p1, url=url)
+            elif '@' in p2:
+                return cls(name=p1, email=p2)
+            return cls(text)
+        elif '<' in text:
             (name, email) = parseaddr(text)
             return cls(name=name, email=email)
         else:
